@@ -32,6 +32,7 @@
 #include "net/SocketUtil.hxx"
 #include "system/Error.hxx"
 #include "fs/AllocatedPath.hxx"
+#include "fs/StandardDirectory.hxx"
 #include "fs/XDG.hxx"
 #include "util/Domain.hxx"
 #include "util/RuntimeError.hxx"
@@ -85,13 +86,10 @@ ListenXdgRuntimeDir(ClientListener &listener) noexcept
 		   use $XDG_RUNTIME_DIR */
 		return false;
 
-	Path xdg_runtime_dir = Path::FromFS(getenv("XDG_RUNTIME_DIR"));
-	if (xdg_runtime_dir.IsNull())
+	const auto mpd_runtime_dir = GetAppRuntimeDir();
+	if (mpd_runtime_dir.IsNull())
 		return false;
 
-	const auto mpd_runtime_dir = xdg_runtime_dir / Path::FromFS("mpd");
-	mkdir(mpd_runtime_dir.c_str(), 0700);
-
 	const auto socket_path = mpd_runtime_dir / Path::FromFS("socket");
 	unlink(socket_path.c_str());
 
