@@ -23,10 +23,13 @@
 #include "ReplayGainInfo.hxx"
 #include "ReplayGainConfig.hxx"
 #include "mixer/MixerControl.hxx"
+#include "mixer/MixerInternal.hxx"
+#include "mixer/Listener.hxx"
 #include "pcm/AudioFormat.hxx"
 #include "pcm/Volume.hxx"
 #include "util/ConstBuffer.hxx"
 #include "util/Domain.hxx"
+#include "Idle.hxx"
 #include "Log.hxx"
 
 #include <cassert>
@@ -94,9 +97,9 @@ public:
 			/* no change */
 			return;
 
-		FormatDebug(replay_gain_domain,
-			    "replay gain mode has changed %s->%s\n",
-			    ToString(mode), ToString(_mode));
+		FmtDebug(replay_gain_domain,
+			 "replay gain mode has changed {}->{}",
+			 ToString(mode), ToString(_mode));
 
 		mode = _mode;
 		Update();
@@ -155,8 +158,7 @@ ReplayGainFilter::Update()
 	if (mode != ReplayGainMode::OFF) {
 		const auto &tuple = info.Get(mode);
 		float scale = tuple.CalculateScale(config);
-		FormatDebug(replay_gain_domain,
-			    "scale=%f\n", (double)scale);
+		FmtDebug(replay_gain_domain, "scale={}\n", scale);
 
 		volume = pcm_float_to_volume(scale);
 	}
@@ -170,6 +172,12 @@ ReplayGainFilter::Update()
 
 		try {
 			mixer_set_volume(mixer, _volume);
+
+			/* invoke the mixer's listener manually, just
+			   in case the mixer implementation didn't do
+			   that already (this depends on the
+			   implementation) */
+			mixer->listener.OnMixerVolumeChanged(*mixer, _volume);
 		} catch (...) {
 			LogError(std::current_exception(),
 				 "Failed to update hardware mixer");
