@@ -20,7 +20,6 @@
 #include "Charset.hxx"
 #include "Features.hxx"
 #include "Domain.hxx"
-#include "Log.hxx"
 #include "lib/icu/Converter.hxx"
 #include "util/AllocatedString.hxx"
 #include "config.h"
@@ -45,11 +44,9 @@ SetFSCharset(const char *charset)
 	assert(charset != nullptr);
 	assert(fs_converter == nullptr);
 
+	fs_charset = charset;
 	fs_converter = IcuConverter::Create(charset);
 	assert(fs_converter != nullptr);
-
-	FmtDebug(path_domain,
-		 "SetFSCharset: fs charset is {}", fs_charset);
 }
 
 #endif
