@@ -61,7 +61,7 @@ public:
 	CurlSocket(const CurlSocket &) = delete;
 	CurlSocket &operator=(const CurlSocket &) = delete;
 
-	auto &GetEventLoop() const noexcept {
+	[[nodiscard]] auto &GetEventLoop() const noexcept {
 		return socket_event.GetEventLoop();
 	}
 
@@ -73,7 +73,7 @@ public:
 				  void *userp, void *socketp) noexcept;
 
 private:
-	SocketDescriptor GetSocket() const noexcept {
+	[[nodiscard]] SocketDescriptor GetSocket() const noexcept {
 		return socket_event.GetSocket();
 	}
 
