@@ -45,7 +45,7 @@ SmbclientContext::New()
 	SMBCCTX *ctx;
 
 	{
-		const std::lock_guard<Mutex> protect(global_mutex);
+		const std::scoped_lock<Mutex> protect(global_mutex);
 		ctx = smbc_new_context();
 	}
 
