@@ -21,12 +21,12 @@
 #include "FlacAudioFormat.hxx"
 #include "ScanVorbisComment.hxx"
 #include "pcm/CheckAudioFormat.hxx"
-#include "MixRampInfo.hxx"
 #include "tag/Handler.hxx"
 #include "tag/Builder.hxx"
 #include "tag/Tag.hxx"
 #include "tag/ReplayGain.hxx"
-#include "tag/MixRamp.hxx"
+#include "tag/MixRampInfo.hxx"
+#include "tag/MixRampParser.hxx"
 #include "ReplayGainInfo.hxx"
 #include "util/StringView.hxx"
 
